"""
SPARQL query related functions
Adapted from: 
https://github.com/SciCrunch/sparc-curation/blob/master/docs/sckan-python.ipynb
"""
import io
import csv
import requests
from urllib.parse import quote as url_quote

def procq(res):
    _, (str_count,) = res
    return int(str_count)

def sparql_query(query, *, endpoint, **kwargs):
    qq = url_quote(query, safe='')
    url = f'{endpoint}?query={qq}'
    headers = {'Accept': 'text/csv'}
    resp = requests.get(url, headers=headers)
    return list(csv.reader(io.StringIO(resp.text)))


example_query = """
PREFIX owl: <http://www.w3.org/2002/07/owl#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX partOf: <http://purl.obolibrary.org/obo/BFO_0000050>
PREFIX ilxtr: <http://uri.interlex.org/tgbugs/uris/readable/>
PREFIX oboInOwl: <http://www.geneontology.org/formats/oboInOwl#> 

SELECT DISTINCT ?Neuron_IRI ?Neuron_Label ?A ?Region_A ?B ?Region_B ?C ?Region_C ?Species ?Species_link
{

    ?Neuron_IRI rdfs:label ?Neuron_Label;
                ilxtr:hasSomaLocation ?A;
                ilxtr:hasAxonLocation ?C;
                (ilxtr:hasAxonTerminalLocation | ilxtr:hasAxonSensoryLocation) ?B. 

    ?Neuron_IRI ilxtr:isObservedInSpecies ?Species_link.

    ?A (rdfs:label | oboInOwl:hasExactSynonym) ?Region_A.
    ?B (rdfs:label | oboInOwl:hasExactSynonym) ?Region_B.
    ?C (rdfs:label | oboInOwl:hasExactSynonym) ?Region_C.
    ?Species_link (rdfs:label | oboInOwl:hasExactSynonym) ?Species.

}
ORDER BY ?Neuron_IRI ?Region_A ?Region_B ?Region_C ?Species
"""


neuron_path_query = """
PREFIX owl: <http://www.w3.org/2002/07/owl#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX partOf: <http://purl.obolibrary.org/obo/BFO_0000050>
PREFIX ilxtr: <http://uri.interlex.org/tgbugs/uris/readable/>
PREFIX oboInOwl: <http://www.geneontology.org/formats/oboInOwl#> 

SELECT DISTINCT ?Neuron_IRI ?Neuron_Label ?A ?Region_A ?B ?Region_B ?C ?Region_C ?Species ?Species_link
{

    ?Neuron_IRI rdfs:label ?Neuron_Label;
                ilxtr:hasSomaLocation ?A;
                ilxtr:hasAxonLocation ?C;
                (ilxtr:hasAxonTerminalLocation | ilxtr:hasAxonSensoryLocation) ?B. 

    ?Neuron_IRI ilxtr:isObservedInSpecies ?Species_link.

    ?A (rdfs:label | oboInOwl:hasExactSynonym) ?Region_A.
    ?B (rdfs:label | oboInOwl:hasExactSynonym) ?Region_B.
    ?C (rdfs:label | oboInOwl:hasExactSynonym) ?Region_C.
    ?Species_link (rdfs:label | oboInOwl:hasExactSynonym) ?Species.

}
ORDER BY ?Neuron_IRI ?Region_A ?Region_B ?Region_C ?Species
"""

neuron_path_phenotype_query = """
PREFIX owl: <http://www.w3.org/2002/07/owl#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX partOf: <http://purl.obolibrary.org/obo/BFO_0000050>
PREFIX ilxtr: <http://uri.interlex.org/tgbugs/uris/readable/>
PREFIX oboInOwl: <http://www.geneontology.org/formats/oboInOwl#> 

SELECT DISTINCT ?Neuron_IRI ?Neuron_Label ?Species ?phenotype ?A ?B ?C ?Region_A ?Region_B ?Region_C
{

    ?Neuron_IRI rdfs:label ?Neuron_Label.

    ?Neuron_IRI rdfs:label ?Neuron_Label;
                    ilxtr:hasSomaLocation ?A;
                    ilxtr:hasAxonLocation ?C;
                    (ilxtr:hasAxonTerminalLocation | ilxtr:hasAxonSensoryLocation) ?B. 

    ?Neuron_IRI ilxtr:isObservedInSpecies ?Species_link.
    ?Neuron_IRI ilxtr:hasNeuronalPhenotype ?phenotype_link.

    ?Species_link (rdfs:label | oboInOwl:hasExactSynonym) ?Species.
    ?phenotype_link (rdfs:label | oboInOwl:hasExactSynonym) ?phenotype.
    ?A (rdfs:label | oboInOwl:hasExactSynonym) ?Region_A.
    ?B (rdfs:label | oboInOwl:hasExactSynonym) ?Region_B.
    ?C (rdfs:label | oboInOwl:hasExactSynonym) ?Region_C.

}
ORDER BY ?Neuron_IRI ?Neuron_Label ?Species ?phenotype ?A ?B ?C ?Region_A ?Region_B ?Region_C
"""

neuron_circuit_role_query = """
PREFIX owl: <http://www.w3.org/2002/07/owl#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX partOf: <http://purl.obolibrary.org/obo/BFO_0000050>
PREFIX ilxtr: <http://uri.interlex.org/tgbugs/uris/readable/>
PREFIX oboInOwl: <http://www.geneontology.org/formats/oboInOwl#> 

SELECT DISTINCT ?Neuron_IRI ?Neuron_Label ?Species ?phenotype ?A ?B ?C ?Region_A ?Region_B ?Region_C
{

    ?Neuron_IRI rdfs:label ?Neuron_Label.

    ?Neuron_IRI rdfs:label ?Neuron_Label;
                    ilxtr:hasSomaLocation ?A;
                    ilxtr:hasAxonLocation ?C;
                    (ilxtr:hasAxonTerminalLocation | ilxtr:hasAxonSensoryLocation) ?B. 

    ?Neuron_IRI ilxtr:isObservedInSpecies ?Species_link.
    ?Neuron_IRI ilxtr:hasCircuitRole ?phenotype_link.

    ?Species_link (rdfs:label | oboInOwl:hasExactSynonym) ?Species.
    ?phenotype_link (rdfs:label | oboInOwl:hasExactSynonym) ?phenotype.
    ?A (rdfs:label | oboInOwl:hasExactSynonym) ?Region_A.
    ?B (rdfs:label | oboInOwl:hasExactSynonym) ?Region_B.
    ?C (rdfs:label | oboInOwl:hasExactSynonym) ?Region_C.

}
ORDER BY ?Neuron_IRI ?Neuron_Label ?Species ?phenotype ?A ?B ?C ?Region_A ?Region_B ?Region_C
"""

projection_fibres_query = """
SELECT DISTINCT ?Neuron_1_IRI ?Neuron_1_Label
                ?Neuron_2_IRI ?Neuron_2_Label ?Species
WHERE
{
    ?Neuron_1_IRI ilxtr:hasSomaLocation ?Neuron_1_A;
                (ilxtr:hasAxonTerminalLocation | ilxtr:hasAxonSensoryLocation) ?Neuron_1_B .

    ?Neuron_2_IRI ilxtr:hasSomaLocation ?Neuron_2_A;
                (ilxtr:hasAxonTerminalLocation | ilxtr:hasAxonSensoryLocation) ??Neuron_2_B .

    ?Neuron_IRI ilxtr:isObservedInSpecies ?Species_link.
    ?Neuron_1_IRI ilxtr:hasForwardConnection ?Neuron_2_IRI .

    ?Neuron_1_IRI rdfs:label ?Neuron_1_Label .
    ?Neuron_2_IRI rdfs:label ?Neuron_2_Label .

    ?Species_link (rdfs:label | oboInOwl:hasExactSynonym) ?Species.
}
"""

app_query = """
PREFIX owl: <http://www.w3.org/2002/07/owl#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX partOf: <http://purl.obolibrary.org/obo/BFO_0000050>
PREFIX ilxtr: <http://uri.interlex.org/tgbugs/uris/readable/>
PREFIX oboInOwl: <http://www.geneontology.org/formats/oboInOwl#> 

SELECT DISTINCT ?Neuron_IRI ?Neuron_Label ?A ?Region_A ?B ?Region_B ?C ?Region_C ?Species ?Species_link
{

   ?Neuron_IRI rdfs:label ?Neuron_Label;
               ilxtr:hasSomaLocation ?A;
               ilxtr:hasAxonLocation ?C;
              (ilxtr:hasAxonTerminalLocation | ilxtr:hasAxonSensoryLocation) ?B. 
   
   ?Neuron_IRI ilxtr:isObservedInSpecies/rdfs:label ?Species.
   ?Neuron_IRI ilxtr:isObservedInSpecies ?Species_link.
    
    ?A (rdfs:label | oboInOwl:hasExactSynonym) ?Region_A.
    ?B (rdfs:label | oboInOwl:hasExactSynonym) ?Region_B.
    ?C (rdfs:label | oboInOwl:hasExactSynonym) ?Region_C.
    ?Species_link (rdfs:label | oboInOwl:hasExactSynonym) ?Species.
    
}
ORDER BY ?Neuron_IRI ?Region_A ?Region_B ?Region_C ?Species
"""


import numpy as np
import plotly.graph_objects as go
from PIL import Image

class SimpleVisualizer(object):

    def __init__(self):
        self.SCALE = 150
        self.MAX_Y = 900

        node_A = Image.open("/home/shailesh/gits/2023-team-4/sckan_compare/data/node_A.png")
        node_B = Image.open("/home/shailesh/gits/2023-team-4/sckan_compare/data/node_B.png")
        node_C = Image.open("/home/shailesh/gits/2023-team-4/sckan_compare/data/node_C.png")
        self.icons = {
            "node_A": node_A,
            "node_B": node_B,
            "node_C": node_C
        }

        self.fig = go.FigureWidget()
        self.fig.layout.hovermode = 'closest'
        self.fig.layout.hoverdistance = -1 #ensures no "gaps" for selecting sparse data

    def interpolate_coordinates(self, point1, point2, resolution=0.1):
        """
        Interpolate between two cartesian coordinates with a given resolution using NumPy.
        
        Parameters:
            point1 (tuple): First cartesian coordinate (x1, y1).
            point2 (tuple): Second cartesian coordinate (x2, y2).
            resolution (float): Interpolation resolution.
            
        Returns:
            tuple: Two lists of interpolated x and y coordinates.
        """
        x1, y1 = point1
        x2, y2 = point2
        
        num_steps = int(1 / resolution)
        t = np.linspace(0, 1, num_steps + 1)
        
        interpolated_x = x1 + (x2 - x1) * t
        interpolated_y = y1 + (y2 - y1) * t
        
        return list(interpolated_x), list(interpolated_y)    

    def plot_figure(self, df, region_A, region_B):
        req_df = df[(df.Region_A == region_A) & (df.Region_B == region_B)]
        list_region_C = req_df.Region_C.unique()

        self.MAX_X = self.SCALE * len(list_region_C)
        self.update_graph()

        # region_A background
        self.draw_block_bg(
            x0=(self.MAX_X/2)-100,
            y0=50,
            x1=(self.MAX_X/2)+100,
            y1=200,
            opacity=0.2,
            color="PaleTurquoise"
        )
        # region A icon
        self.draw_image(
            icon=self.icons["node_A"],
            x=self.MAX_X/2,
            y=125,
        )
        # region A label
        self.add_text(
            x=self.MAX_X/2-3,
            y=0,
            text=region_A
        )
        self.mark_node(x=self.MAX_X/2, y=125, label=region_A)
        
        # region_B background
        self.draw_block_bg(
            x0=(self.MAX_X/2)-100,
            y0=self.MAX_Y-200,
            x1=(self.MAX_X/2)+100,
            y1=self.MAX_Y-50,
            opacity=0.2,
            color="LightGreen"
        )
        # region B icon
        self.draw_image(
            icon=self.icons["node_B"],
            x=self.MAX_X/2-3,
            y=self.MAX_Y-125,
        )
        # region B label
        self.add_text(
            x=self.MAX_X/2,
            y=self.MAX_Y,
            text=region_B
        )
        self.mark_node(x=self.MAX_X/2, y=self.MAX_Y-125, label=region_B)


        # all region C and connections
        for i in range(0,len(list_region_C)):
            xtemp = (self.SCALE*i)+64
            ytemp = self.MAX_Y/2

            self.draw_image(
                icon=self.icons["node_C"],
                x=xtemp,
                y=ytemp,
            )
            self.add_text(
                x=xtemp,
                y=ytemp+100,
                text=req_df.iloc[i,7],
                fontsize=12
            )
            self.mark_node(x=xtemp, y=ytemp, label=req_df.iloc[i,7])
            p1 = (self.MAX_X/2, 125)
            p2 = (xtemp, ytemp)
            x_new, y_new = self.interpolate_coordinates(p1, p2)
            self.draw_connections(
                x=x_new,
                y=y_new,
                label=req_df.iloc[i,1],
                color="#FF0000"
            )
            p1 = (xtemp, ytemp)
            p2 = (self.MAX_X/2, self.MAX_Y-125)
            x_new, y_new = self.interpolate_coordinates(p1, p2)
            self.draw_connections(
                x=x_new,
                y=y_new,
                label=req_df.iloc[i,1],
                color="#FF0000"
            )
            
        return self.fig

    def update_graph(self):
        self.fig.update_xaxes(showgrid=False, zeroline=False, visible=False, showticklabels=False)
        self.fig.update_yaxes(showgrid=False, zeroline=False, visible=False, showticklabels=False)
        self.fig.update_yaxes(range = [self.MAX_Y, 0])
        self.fig.update_xaxes(range = [0, self.MAX_X])
        self.fig.update_layout(height=int(500))
        self.fig.update_layout(template="plotly_white")
        self.fig.update_layout(showlegend=False)


    def draw_block_bg(self, x0, y0, x1, y1, opacity, color):
        self.fig.add_shape(
            type="rect",
            xref="x", yref="y",
            x0=x0,
            y0=y0,
            x1=x1,
            y1=y1,
            opacity=opacity,
            fillcolor=color,
            line_color=color,
        )

    def draw_image(self, icon, x, y):
        self.fig.add_layout_image(
            dict(
                source=icon,
                xref="x",
                yref="y",
                xanchor="center",
                yanchor="middle",
                x=x,
                y=y,
                sizex=128,
                sizey=128,
                opacity=1.0,
                layer="above"
            )
        )

    def draw_connections(self, x, y, label, color):
        self.fig.add_trace(
            go.Scatter(
                x=x,
                y=y,
                mode="lines",
                line_color=color,
                showlegend=False,
                text=label,
                hoverinfo="text",
                name=label,
            )
        )

    def add_text(self, x, y, text, fontsize=20):
        self.fig.add_annotation(
            x=x,
            y=y,
            xref="x",
            yref="y",
            text=text,
            showarrow=False,
            font=dict(
                family="Courier New, monospace",
                size=fontsize,
                color="#000000"
                ),
            align="center",
        )

    def mark_node(self, x, y, label):
        self.fig.add_trace(go.Scatter(
            x=[x],
            y=[y],
            mode = "markers",
            marker_symbol="circle",
            text=label,
            hoverinfo="text",
            name=label,
        ))


import numpy as np
import plotly.graph_objects as go

class Visualizer(object):

    def __init__(self, region_dict, species):
        self.region_dict = region_dict
        self.species = species

        self.SCALE = 50
        self.MAX_X = 43
        self.MAX_Y = 18
        self.NODE_RADIUS = 0.2

        self.fig = go.FigureWidget()
        self.fig.layout.hovermode = 'closest'
        self.fig.layout.hoverdistance = -1 #ensures no "gaps" for selecting sparse data

        self.fig.update_xaxes(showgrid=False, zeroline=False, visible=False, showticklabels=False)
        self.fig.update_yaxes(showgrid=False, zeroline=False, visible=False, showticklabels=False)
        self.fig.update_layout(showlegend=False)
        # # self.fig.update_yaxes(
        # #     scaleanchor="x", scaleratio=1)
        self.fig.update_yaxes(range = [self.MAX_Y+3, 0])
        self.fig.update_xaxes(range = [0, self.MAX_X])
        self.fig.update_layout(height=int(500))

        if (self.species == "Mus musculus") or (self.species == "Rattus norvegicus"):
            self.draw_background_mouse_rat()
        else:
            # default
            self.draw_background_human()


    def draw_rect(self,
                start_x,
                start_y,
                width=1,
                height=1,
                color_border="#4051BF",
                color_fill="#C5CAE9",
                tooltiptext="<set name>"):
        self.fig.add_trace(go.Scatter(
            x=[start_x,start_x+width,start_x+width,start_x, start_x],
            y=[start_y,start_y,start_y+height,start_y+height, start_y],
            mode ="lines",
            fill="toself",
            line_color=color_border,
            fillcolor=color_fill,
            text=tooltiptext,
            hoveron = "fills",
            hoverinfo = "text",
            showlegend=False
        ))

    def draw_poly(self,
                xlist,
                ylist,
                color_border="#4051BF",
                color_fill="#C5CAE9",
                tooltiptext="<set name>"):
        self.fig.add_trace(go.Scatter(
            x=xlist,
            y=ylist,
            mode ="lines",
            fill="toself",
            line_color=color_border,
            fillcolor=color_fill,
            text=tooltiptext,
            hoveron = "fills",
            hoverinfo = "text",
            showlegend=False
        ))

    def draw_background_human(self):
        self.draw_rect(1, 3, 4, 4, "#4051BF", "#C5CAE9")
        self.draw_rect(5, 3, 1, 4, "#4051BF", "#C5CAE9")
        self.draw_rect(6, 5, 2, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(7, 4, 2, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(8, 5, 2, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(10, 2, 31, 1, "#4051BF", "#C5CAE9")        
        self.draw_rect(10, 3, 8, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(18, 3, 12, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(30, 3, 5, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(35, 3, 5, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(40, 3, 1, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(10, 5, 31, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(10, 6, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(13, 6, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(17, 6, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(18, 6, 23, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(10, 7, 3, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(35, 7, 3, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(16, 8, 2, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(19, 8, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_poly([6,8,8,7,7,6,6], [9,9,11,11,10,10,9], "#4051BF", "#C5CAE9")
        self.draw_rect(18, 9, 3, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(21, 9, 2, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(29, 13, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(18, 11, 10, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(29, 11, 12, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(22, 13, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(18, 14, 23, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(37, 16, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(40, 16, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(35, 17, 6, 2, "#4051BF", "#C5CAE9")

    def draw_background_mouse_rat(self):
        self.draw_rect(24, 2, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(26, 2, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(1, 3, 4, 4, "#4051BF", "#C5CAE9")
        self.draw_rect(5, 3, 1, 4, "#4051BF", "#C5CAE9")
        self.draw_rect(6, 5, 2, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(7, 4, 2, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(8, 5, 2, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(10, 3, 8, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(18, 3, 13, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(31, 3, 6, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(37, 3, 4, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(41, 3, 1, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(10, 5, 31, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(10, 6, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(13, 6, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(17, 6, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(18, 6, 23, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(10, 7, 3, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(35, 7, 3, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(6, 8, 2, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(15, 8, 2, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(19, 8, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_poly([6,8,8,7,7,6,6], [9,9,11,11,10,10,9], "#4051BF", "#C5CAE9")
        self.draw_rect(18, 9, 3, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(21, 9, 2, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(29, 13, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(40, 10, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(18, 11, 10, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(29, 11, 12, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(22, 13, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(24, 13, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(18, 14, 23, 2, "#4051BF", "#C5CAE9")
        self.draw_rect(37, 16, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(40, 16, 1, 1, "#4051BF", "#C5CAE9")
        self.draw_rect(35, 17, 6, 2, "#4051BF", "#C5CAE9")


    def mark_node(self,
                  region,
                  color_border="#FF0000",
                  color_fill="#FFFF00",
                  small=False):
        x = self.region_dict[region][0] + 0.5
        y = self.region_dict[region][1] + 0.5
        size_factor = 7 if small else 5
        self.fig.add_trace(go.Scatter(
            x=[x],
            y=[y],
            mode = 'markers',
            marker_symbol = 'circle',

            marker_size = int(self.SCALE/size_factor),
            marker=dict(
                color=color_fill,
                line=dict(
                    color=color_border,
                    width=2
                )
            ),
            text=region,
            hoverinfo="text",
            name=region
        ))

    def interpolate_coordinates(self, point1, point2, resolution=0.1):
        """
        Interpolate between two cartesian coordinates with a given resolution using NumPy.
        
        Parameters:
            point1 (tuple): First cartesian coordinate (x1, y1).
            point2 (tuple): Second cartesian coordinate (x2, y2).
            resolution (float): Interpolation resolution.
            
        Returns:
            tuple: Two lists of interpolated x and y coordinates.
        """
        x1, y1 = point1
        x2, y2 = point2
        
        num_steps = int(1 / resolution)
        t = np.linspace(0, 1, num_steps + 1)
        
        interpolated_x = x1 + (x2 - x1) * t
        interpolated_y = y1 + (y2 - y1) * t
        
        return list(interpolated_x), list(interpolated_y)

    def draw_edge_AB(self,
                     region1,
                     region2,
                     neuron=None):
        # From A to B
        default_linewidth = 2
        x1, y1 = self.region_dict[region1]
        x2, y2 = self.region_dict[region2]
        self.mark_node(region1)
        self.mark_node(region2)
        # self.fig.add_trace(go.Scatter(
        #     x=[x1+0.5, x2+0.5],
        #     y=[y1+0.5, y2+0.5],
        #     mode = 'lines',
        #     text=neuron,
        #     meta="main",
        #     hoverinfo="text",
        #     name=neuron,
        #     line_color="#FF0000",
        #     line={"width":default_linewidth}
        # ))

        # interpolate line for adding hover text on line (to display neuron name)
        p1 = (x1+0.5, y1+0.5)
        p2 = (x2+0.5, y2+0.5)
        x_new, y_new = self.interpolate_coordinates(p1, p2)
        self.fig.add_trace(go.Scatter(
            x=x_new,
            y=y_new,
            mode = 'lines',
            text=neuron,
            hoverinfo="text",
            name=neuron,
            line_color="#FF0000",
            line={"width":default_linewidth},
            showlegend=False
        ))
        # ax.plot((x1, x2), (y1, y2), linewidth=2, color='firebrick')

    def draw_edge_ABC(self,
                      region1,
                      region2,
                      region3,
                      neuron=None):
        # From A to B via C
        default_linewidth = 2
        x1, y1 = self.region_dict[region1]
        x2, y2 = self.region_dict[region2]
        x3, y3 = self.region_dict[region3]
        self.mark_node(region1)
        self.mark_node(region2)
        self.mark_node(region3, color_border="#000000", color_fill="#00FF00", small=True)
        # self.fig.add_trace(go.Scatter(
        #     x=[x1+0.5, x3+0.5, x2+0.5],
        #     y=[y1+0.5, y3+0.5, y2+0.5],
        #     mode = 'lines',
        #     text=neuron,
        #     hoverinfo="text",
        #     name=neuron,
        #     # line_shape="spline",
        #     line_color="#FF0000",
        #     line={"width":default_linewidth}
        # ))

        # interpolate line for adding hover text on line (to display neuron name)
        # first part of line: A to C
        p1 = (x1+0.5, y1+0.5)
        p2 = (x3+0.5, y3+0.5)
        x_new, y_new = self.interpolate_coordinates(p1, p2)
        self.fig.add_trace(go.Scatter(
            x=x_new,
            y=y_new,
            mode = 'lines',
            text=neuron,
            hoverinfo="text",
            name=neuron,
            # line_shape="spline",
            line_color="#FF0000",
            line={"width":default_linewidth},
            showlegend=False
        ))

        # second part of line: C to B
        p1 = (x3+0.5, y3+0.5)
        p2 = (x2+0.5, y2+0.5)
        x_new, y_new = self.interpolate_coordinates(p1, p2)
        self.fig.add_trace(go.Scatter(
            x=x_new,
            y=y_new,
            mode = 'lines',
            text=neuron,
            hoverinfo="text",
            name=neuron,
            # line_shape="spline",
            line_color="#FF0000",
            line={"width":default_linewidth},
            showlegend=False
        ))

    def show_figure(self):
        self.fig.show(config= {'displaylogo': False})

    def get_figure(self):
        return self.fig



"""
A package for retrieving and visualizing data contained in SCKAN
(e.g., across species, relationship to spinal segments) to highlight
similarities and differences in neuronal pathways

License: Apache License 2.0
"""

import os
import json
import pkg_resources
import numpy as np
import pandas as pd

from . import globals
from . import query
from .visualize import Visualizer


class SckanCompare(object):
    """
    Base class for accessing functionality
    """

    def __init__(self, species="Homo sapiens", endpoint=globals.BLAZEGRAPH_ENDPOINT):
        self.endpoint = endpoint
        self.region_dict = {}
        self.vis = None
        self.species = species
        self.load_json_file()

    def execute_query(self, query_string):
        # execute specified SPAQRL query and return result
        return query.sparql_query(query_string, endpoint=self.endpoint)


    def load_json_file(self):
        datapath = pkg_resources.resource_filename("sckan_compare", "data")
        if self.species == "Mus musculus":
            filepath = os.path.join(datapath, "coords_mouse.json")
        elif self.species == "Rattus norvegicus":
            filepath = os.path.join(datapath, "coords_rat.json")
        else:
            # default
            filepath = os.path.join(datapath, "coords_human.json")

        with open(filepath, encoding='utf-8-sig') as json_file:
            data = json.load(json_file)

        self.region_dict[self.species] = {}
        for item in data:
            self.region_dict[self.species][item["Name"]] = [int(item["X"]), int(item["Y"])]
   
    def reset_vis(self):
        self.vis = Visualizer(self.region_dict[self.species], self.species)
        return self.vis

    def add_connection(self, region_A=None, region_B=None, region_C=None, neuron=None):
        if not self.vis:
            self.vis = Visualizer(self.region_dict[self.species], self.species)

        if not region_A:
            raise ValueError("region_A needs to be specified!")
        if not region_B:
            raise ValueError("region_B needs to be specified!")

        if region_C:
            # A->C->B
            self.vis.draw_edge_ABC(region_A, region_B, region_C, neuron)
        else:
            # A->B
            self.vis.draw_edge_AB(region_A, region_B, neuron)


    def get_graph(self):
        return self.vis.get_figure()
    

    def get_unique_region_IRI(self, df_result):
        """
        :param df_result: dataframe
        :return: unique regions in form of hyperlinks
        """
        region_A = np.unique(df_result.loc[:,"A"])
        region_B = np.unique(df_result.loc[:,"B"])
        region_C = np.unique(df_result.loc[:,"C"])
        unique_regions = np.unique(np.concatenate((region_A, region_B, region_C)))
        return unique_regions

    
    def get_unique_regions (self, df_result):
        """
        :param df_result: dataframe
        :return: region labels
        """
        unique_regions = self.get_unique_region_IRI(df_result)
        test_template = """
        PREFIX owl: <http://www.w3.org/2002/07/owl#>
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
        PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
        PREFIX partOf: <http://purl.obolibrary.org/obo/BFO_0000050>
        PREFIX ilxtr: <http://uri.interlex.org/tgbugs/uris/readable/>

        SELECT DISTINCT ?annotation ?label 
        WHERE {{
            ?annotation rdfs:label ?label. 

            FILTER (?annotation IN ({}))
        }}
        ORDER BY ?annotation ?label 
        """

        # first 100
        formatted_links = " ,".join(["<{}>".format(link) for link in unique_regions[:100]])
        test_query = test_template.format(formatted_links)
        result = self.execute_query(test_query)
        final_df = pd.DataFrame(result).loc[1:, :]

        # rest 100
        formatted_links = " ,".join(["<{}>".format(link) for link in unique_regions[100:200]])
        test_query = test_template.format(formatted_links)
        result = self.execute_query(test_query)
        final_df = pd.concat([final_df, pd.DataFrame(result).loc[1:, :]], ignore_index=True)

        # rest all
        formatted_links = " ,".join(["<{}>".format(link) for link in unique_regions[200:]])
        test_query = test_template.format(formatted_links)
        result = self.execute_query(test_query)
        final_df = pd.concat([final_df, pd.DataFrame(result).loc[1:, :]], ignore_index=True)

        final_df.drop_duplicates(subset=0, keep='first', inplace=True)
        return final_df
    
    # getting dataframe from queries
    def get_neuron_dataframe(self, result, species_name=None, phenotype_name=None):
        """
        :param result: from queries
        :param species_name: takes only subset for circuit_role (check get_circuit_roles), all from get_species_name
        :param phenotype_name: from get_phenotype_neuron or from get_circuit_roles
        :return:
        """

        df_result = pd.DataFrame(result)
        # get column names right
        df_result.columns = df_result.loc[0, :].to_list()
        df_result = df_result.loc[1:, :]

        # replace duplicate instances of species name
        df_result = df_result.replace('Mammalia', 'Mammal')
        df_result = df_result.replace('mammals', 'Mammal')
        df_result = df_result.replace('Vertebrata <vertebrates>', 'Vertebrata')
        df_result = df_result.replace('vertebrates', 'Vertebrata')
        df_result = df_result.replace('human', 'Homo sapiens')
        df_result = df_result.replace('Norway rat', 'Rattus norvegicus')
        df_result = df_result.replace('brown rat', 'Rattus norvegicus')
        df_result = df_result.replace('rat', 'Rattus norvegicus')
        df_result = df_result.replace('rats', 'Rattus norvegicus')
        df_result = df_result.replace('mouse', 'Mus musculus')
        df_result = df_result.replace('house mouse', 'Mus musculus')

        if phenotype_name:

            if species_name:
                filtered_df = df_result[(df_result.Species == species_name)]
                unique_regions_df = self.get_unique_regions(filtered_df)
                # Create a dictionary with the values from DataFrame 2 as keys and the replacement values as values
                replacement_dict = dict(zip(unique_regions_df[0], unique_regions_df[1]))

                # # Use the map function to replace the values in DataFrame 1
                filtered_df.loc[:, "Region_A"] = filtered_df.A.map(replacement_dict)
                filtered_df.loc[:, "Region_B"] = filtered_df.B.map(replacement_dict)
                filtered_df.loc[:, "Region_C"] = filtered_df.C.map(replacement_dict)

                # remove duplicate
                filtered_df = filtered_df.drop_duplicates()
                filtered_df = filtered_df[(filtered_df.phenotype == phenotype_name)]

                return filtered_df

            else:
                unique_regions_df = self.get_unique_regions(df_result)
                # Create a dictionary with the values from DataFrame 2 as keys and the replacement values as values
                replacement_dict = dict(zip(unique_regions_df[0], unique_regions_df[1]))

                # # Use the map function to replace the values in DataFrame 1
                df_result.loc[:, "Region_A"] = df_result.A.map(replacement_dict)
                df_result.loc[:, "Region_B"] = df_result.B.map(replacement_dict)
                df_result.loc[:, "Region_C"] = df_result.C.map(replacement_dict)

                # remove duplicate
                df_result = df_result.drop_duplicates()
                df_result = df_result[(df_result.phenotype == phenotype_name)]

                return df_result

        else:

            if species_name:
                filtered_df = df_result[(df_result.Species == species_name)]
                unique_regions_df = self.get_unique_regions(filtered_df)
                # Create a dictionary with the values from DataFrame 2 as keys and the replacement values as values
                replacement_dict = dict(zip(unique_regions_df[0], unique_regions_df[1]))

                # # Use the map function to replace the values in DataFrame 1
                filtered_df.loc[:, "Region_A"] = filtered_df.A.map(replacement_dict)
                filtered_df.loc[:, "Region_B"] = filtered_df.B.map(replacement_dict)
                filtered_df.loc[:, "Region_C"] = filtered_df.C.map(replacement_dict)

                # remove duplicate
                filtered_df = filtered_df.drop_duplicates()

                return filtered_df

            else:
                unique_regions_df = self.get_unique_regions(df_result)
                # Create a dictionary with the values from DataFrame 2 as keys and the replacement values as values
                replacement_dict = dict(zip(unique_regions_df[0], unique_regions_df[1]))

                # # Use the map function to replace the values in DataFrame 1
                df_result.loc[:, "Region_A"] = df_result.A.map(replacement_dict)
                df_result.loc[:, "Region_B"] = df_result.B.map(replacement_dict)
                df_result.loc[:, "Region_C"] = df_result.C.map(replacement_dict)

                # remove duplicate
                df_result = df_result.drop_duplicates()

                return df_result

    def projection_fibres_dataframe(self, result, species_name=None):
        """

            :param result: from queries
            :param species_name: takes only subset for circuit_role (check get_circuit_roles), all from get_species_name
            :param phenotype_name: from get_phenotype_neuron or from get_circuit_roles
            :return:
            """

        df_result = pd.DataFrame(result)
        # get column names right
        df_result.columns = df_result.loc[0, :].to_list()
        df_result = df_result.loc[1:, :]

        # replace duplicate instances of species name
        df_result = df_result.replace('Mammalia', 'Mammal')
        df_result = df_result.replace('mammals', 'Mammal')
        df_result = df_result.replace('Vertebrata <vertebrates>', 'Vertebrata')
        df_result = df_result.replace('vertebrates', 'Vertebrata')
        df_result = df_result.replace('human', 'Homo sapiens')
        df_result = df_result.replace('Norway rat', 'Rattus norvegicus')
        df_result = df_result.replace('brown rat', 'Rattus norvegicus')
        df_result = df_result.replace('rat', 'Rattus norvegicus')
        df_result = df_result.replace('rats', 'Rattus norvegicus')
        df_result = df_result.replace('mouse', 'Mus musculus')
        df_result = df_result.replace('house mouse', 'Mus musculus')

        if species_name:
            filtered_df = df_result[(df_result.Species == species_name)]
            unique_regions_df = self.get_unique_regions(filtered_df)
            # Create a dictionary with the values from DataFrame 2 as keys and the replacement values as values
            replacement_dict = dict(zip(unique_regions_df[0], unique_regions_df[1]))

            # # Use the map function to replace the values in DataFrame 1
            filtered_df.loc[:, "Region_A"] = filtered_df.A.map(replacement_dict)
            filtered_df.loc[:, "Region_B"] = filtered_df.B.map(replacement_dict)
            filtered_df.loc[:, "Region_C"] = filtered_df.C.map(replacement_dict)

            # remove duplicate
            filtered_df = filtered_df.drop_duplicates()

            return filtered_df

        else:
            unique_regions_df = self.get_unique_regions(df_result)
            # Create a dictionary with the values from DataFrame 2 as keys and the replacement values as values
            replacement_dict = dict(zip(unique_regions_df[0], unique_regions_df[1]))

            # # Use the map function to replace the values in DataFrame 1
            df_result.loc[:, "Region_A"] = df_result.A.map(replacement_dict)
            df_result.loc[:, "Region_B"] = df_result.B.map(replacement_dict)
            df_result.loc[:, "Region_C"] = df_result.C.map(replacement_dict)

            # remove duplicate
            df_result = df_result.drop_duplicates()

            return df_result

# visualising projections of neurons
import dash
import dash_cytoscape as cyto
import dash_html_components as html

def visualise_projection(result_df):
    
    # getting unique nodes
    a = np.unique(result_df.loc[:,'Neuron_1_Label'])
    b = np.unique(result_df.loc[:,'Neuron_2_Label'])
    unique_nodes= np.unique(np.concatenate((a,b)))
        
    nodes = []
    for node in unique_nodes:
        nodes.append({"data": {"id": node, "label": node}})
    
    # construct connections between these nodes as required
    edges = []
    for i in range(len(result_df)):
        item = {"data": {"source": result_df.iloc[i,0], "target": result_df.iloc[i,1]}}
        edges.append(item)
    
    elements = nodes + edges

    #defines styling for the plot
    default_stylesheet = [
    {
        "selector": "node",
        "style": {
            "width": "mapData(size, 0, 100, 20, 60)",
            "height": "mapData(size, 0, 100, 20, 60)",
            "content": "data(label)",
            "font-size": "10px",
            "text-valign": "center",
            "text-halign": "center",}
    },
        {
            "selector": "edge",
            "style": {
             #'line-style': 'dashed',
            'target-arrow-color': 'black',
            'target-arrow-shape': 'vee',
            'curve-style' : 'straight',}
      }
    ]


    app = dash.Dash(__name__)
    app.layout = html.Div([
        cyto.Cytoscape(
            id='cytoscape',
            elements=elements,
            stylesheet = default_stylesheet,
            style={'width': '100%', 'height': '400px', 'background-image': 'url(./sample_bg.png)'},
            layout={
                # 'name': 'preset'
                'name': 'cose'
                # 'name': 'random'
                #'name': 'circle'
                # 'name': 'grid'
            }
            
        )
    ])
    
    
    if __name__ == '__main__':
        app.run_server(debug=True, port=8051)



"""
Author: Shailesh Appukuttan
Date: 07/08/2023
"""
from setuptools import setup, find_packages

setup(
    name="sckan-compare",
    version="0.0.4",
    description="A package for retrieving and visualizing data contained in     SCKAN (e.g., across species, relationship to spinal segments) to highlight similarities and differences in neuronal pathways",
    author="Shailesh Appukuttan, Hiba Ben Aribi, Pranjal Garg, Gautam Kumar",
    author_email="appukuttan.shailesh@gmail.com",
    license="Apache-2.0",
    package_data={'': ['data/*.json']},
    packages=find_packages(),
)



def get_species_name(projection_fibre=False):
    """
    :return: gets the (unique) name of all species in the dataset
    """
    species = ['Canis familiaris', 'Cavia porcellus', 'Homo sapiens', 'Mammal',
         'Mus musculus', 'Rattus norvegicus']

    if projection_fibre:
        species = ['Canis familiaris', 'Cavia porcellus', 'Homo sapiens', 'Mammal',
       'Mus musculus', 'Rattus norvegicus', 'Vertebrata']

    return species


def get_phenotype_neuron():
    """
    :return: gets unique phenotypes of neurons
    """
    phenotype = ['Enteric phenotype', 'Parasympathetic Post-Ganglionic phenotype',
          'Parasympathetic Pre-Ganglionic phenotype',
          'Parasympathetic phenotype', 'Post ganglionic phenotype',
          'Pre ganglionic phenotype',
          'Sympathetic Post-Ganglionic phenotype',
          'Sympathetic Pre-Ganglionic phenotype', 'Sympathetic phenotype']

    return phenotype

def get_circuit_roles():
    species = ['Homo sapiens', 'Mammal', 'Rattus norvegicus']
    circuit_roles = ['Motor phenotype', 'Sensory phenotype']
    return species, circuit_roles